@@ -2,20 +2,21 @@
  * Graph reduction engine, code generation and execution
  *
  * The Hugs 98 system is Copyright (c) Mark P Jones, Alastair Reid, the
- * Yale Haskell Group, and the Oregon Graduate Institute of Science and
- * Technology, 1994-1999, All rights reserved.  It is distributed as
- * free software under the license in the file "License", which is
- * included in the distribution.
+ * Yale Haskell Group, and the OGI School of Science & Engineering at OHSU,
+ * 1994-2003, All rights reserved.  It is distributed as free software under
+ * the license in the file "License", which is included in the distribution.
  *
  * $RCSfile: machine.c,v $
- * $Revision: 1.6 $
- * $Date: 2001/02/14 00:25:26 $
+ * $Revision: 1.24 $
+ * $Date: 2004/10/22 12:38:45 $
  * ------------------------------------------------------------------------*/
 
 #include "prelude.h"
 #include "storage.h"
 #include "connect.h"
 #include "errors.h"
+#include "char.h"
+#include "opts.h"	/* needed for DEBUG_CODE */
 #include <setjmp.h>
 
 #if OBSERVATIONS
@@ -27,8 +28,6 @@ Bool   debugCode     = FALSE;		/* TRUE => print G-code to screen  */
 #if DEBUG_SHOWSC
 Bool   debugSC	     = FALSE;		/* TRUE => print SC-code to screen */
 #endif
-Bool   failOnError   = TRUE;		/* TRUE => abort as soon as error  */
-					/*	   occurs		   */
 
 /* --------------------------------------------------------------------------
  * Data structures for machine memory (program storage):
@@ -41,7 +40,7 @@ Bool   failOnError   = TRUE;		/* TRUE => abort as soon as error  */
  */
 
 #define INSTRLIST       Ins(iLOAD),  Ins(iCELL),   Ins(iCHAR),    \
-			Ins(iINT),   Ins(iFLOAT),  Ins(iSTRING),  \
+			Ins(iINT),   Ins(iDOUBLE), Ins(iSTRING),  \
 			Ins(iMKAP),  Ins(iUPDATE), Ins(iUPDAP),   \
 			Ins(iEVAL),  Ins(iRETURN), Ins(iTEST),    \
 			Ins(iGOTO),  Ins(iSETSTK), Ins(iROOT),    \
@@ -69,6 +68,22 @@ typedef union {
 
 typedef MemCell far *Memory;
 static  Memory      memory;
+#if !WANT_FIXED_SIZE_TABLES
+/* (Dynamically) growable memory. */
+DynTable* dynMemory = NULL;
+#endif
+
+#if !WANT_FIXED_SIZE_TABLES
+void
+growMemory(void)
+{
+    growDynTable(dynMemory);
+    memory = (Memory)(dynMemory->data);
+    if (memory==0)
+	fatal("Cannot allocate program memory");
+}
+#endif
+
 #define intAt(m)    memory[m].mint
 #if !BREAK_FLOATS
 #define floatAt(m)  memory[m].mfloat
@@ -87,12 +102,19 @@ Addr m; {
     return intAt(m);
 }
 
-#if !BREAK_FLOATS
 Float FloatAt(m)
 Addr m; {
+#if BREAK_FLOATS
+    return floatFromParts(cellAt(m),cellAt(m+1));
+#else
     return floatAt(m);
-}
 #endif
+}
+
+Double DoubleAt(m)
+Addr m; {
+    return doubleFromParts(cellAt(m),cellAt(m+1));
+}
 
 Cell CellAt(m)
 Addr m; {
@@ -122,7 +144,7 @@ Addr m; {
 
 static Void  local instrNone    Args((Instr));
 static Void  local instrInt     Args((Instr,Int));
-static Void  local instrFloat   Args((Instr,FloatPro));
+static Void  local instrDouble  Args((Instr,DoublePro));
 static Void  local instrCell    Args((Instr,Cell));
 static Void  local instrText    Args((Instr,Text));
 static Void  local instrLab     Args((Instr,Label));
@@ -145,7 +167,7 @@ static Addr  local dissInstr    Args((Addr));
 static Void  local printCell    Args((Cell));
 static Addr  local dissNone     Args((Addr,String));
 static Addr  local dissInt      Args((Addr,String));
-static Addr  local dissFloat    Args((Addr,String));
+static Addr  local dissDouble   Args((Addr,String));
 static Addr  local dissCell     Args((Addr,String));
 static Addr  local dissText     Args((Addr,String));
 static Addr  local dissAddr     Args((Addr,String));
@@ -186,6 +208,8 @@ static Void  local evalString   Args((Cell));
 static Addr  startInstr;                /* first instruction after START   */
 static Addr  lastInstr;                 /* last instr written (for peephole*/
 					/* optimisations etc.)             */
+static Bool  newBasicBlock;		/* lastInstr ends a basic block    */
+					/* (so no peeping at lastInstr)    */
 static Addr  noMatch;                   /* address of a single FAIL instr  */
 static Int   srsp;                      /* simulated runtime stack pointer */
 static Int   offsPosn[NUM_OFFSETS];     /* mapping from logical to physical*/
@@ -195,6 +219,7 @@ static Void local instrNone(opc)        /* Opcode with no operands         */
 Instr opc; {
     lastInstr          = getMem(1);
     instrAt(lastInstr) = opc;
+    newBasicBlock      = FALSE;
 }
 
 static Void local instrInt(opc,n)       /* Opcode with integer operand     */
@@ -203,21 +228,17 @@ Int   n; {
     lastInstr          = getMem(2);
     instrAt(lastInstr) = opc;
     intAt(lastInstr+1) = n;
+    newBasicBlock      = FALSE;
 }
 
-static Void local instrFloat(opc,fl)    /* Opcode with Float operand       */
+static Void local instrDouble(opc,fl)    /* Opcode with Double operand     */
 Instr    opc;
-FloatPro fl; {
-#if BREAK_FLOATS
+DoublePro fl; {
     lastInstr            = getMem(3);
     instrAt(lastInstr)   = opc;
-    cellAt(lastInstr+1)  = part1Float(fl);
-    cellAt(lastInstr+2)  = part2Float(fl);
-#else
-    lastInstr            = getMem(2);
-    instrAt(lastInstr)   = opc;
-    floatAt(lastInstr+1) = (Float) fl;
-#endif
+    cellAt(lastInstr+1)  = part1Double(fl);
+    cellAt(lastInstr+2)  = part2Double(fl);
+    newBasicBlock        = FALSE;
 }
 
 static Void local instrCell(opc,c)      /* Opcode with Cell operand        */
@@ -226,6 +247,7 @@ Cell  c; {
     lastInstr           = getMem(2);
     instrAt(lastInstr)  = opc;
     cellAt(lastInstr+1) = c;
+    newBasicBlock       = FALSE;
 }
 
 static Void local instrText(opc,t)      /* Opcode with Text operand        */
@@ -234,6 +256,7 @@ Text  t; {
     lastInstr           = getMem(2);
     instrAt(lastInstr)  = opc;
     textAt(lastInstr+1) = t;
+    newBasicBlock       = FALSE;
 }
 
 static Void local instrLab(opc,l)       /* Opcode with label operand       */
@@ -244,6 +267,7 @@ Label l; {
     labAt(lastInstr+1) = l;
     if (l<0)
 	internal("bad Label");
+    newBasicBlock      = FALSE;
 }
 
 static Void local instrCellLab(opc,c,l) /* Opcode with cell, label operands*/
@@ -256,6 +280,7 @@ Label l; {
     labAt(lastInstr+2)  = l;
     if (l<0)
 	internal("bad Label");
+    newBasicBlock       = FALSE;
 }
 
 /* --------------------------------------------------------------------------
@@ -274,7 +299,7 @@ static  Label         nextLab;         /* next label number to allocate    */
 #define UPDRET        (-3)
 #define VALRET        (-4)
 static  Addr          fixups[NUM_FIXUPS]; /* fixup table maps Label -> Addr*/
-#define atLabel(n)    fixups[n] = getMem(0)
+#define atLabel(n)    (newBasicBlock = TRUE, fixups[n] = getMem(0))
 #define endLabel(d,l) if (d==RUNON) atLabel(l)
 #define fix(a)        setAddrAt(a,fixups[labAt(a)])
 
@@ -286,6 +311,7 @@ static Void local asSTART() {          /* initialise assembler             */
     lengthAddr  = getMem(1);
     startInstr  = getMem(0);
     lastInstr   = startInstr-1;
+    newBasicBlock = TRUE;
     srsp        = 0;
     offsPosn[0] = 0;
 }
@@ -326,19 +352,14 @@ static Void local asEND() {            /* Fix addresses in assembled code  */
 	    case iCELL   : 
 	    case iCHAR   :
 	    case iINT    :
-#if !BREAK_FLOATS
-	    case iFLOAT  :
-#endif
 	    case iSTRING :
 	    case iMKAP   :
 	    case iUPDATE :
 	    case iUPDAP  : pc+=2;
 			   break;
 
-#if BREAK_FLOATS
-	    case iFLOAT  : pc+=3;
+	    case iDOUBLE : pc+=3;
 			   break;
-#endif
 
 	    case iTEST   : fix(pc+2);
 			   pc+=3;
@@ -353,7 +374,7 @@ static Void local asEND() {            /* Fix addresses in assembled code  */
  * ------------------------------------------------------------------------*/
 
 #define asINTEGER(n) instrInt(iINT,n);          srsp++
-#define asFLOAT(fl)  instrFloat(iFLOAT,fl);     srsp++
+#define asDOUBLE(fl) instrDouble(iDOUBLE,fl);   srsp++
 #define asSTRING(t)  instrText(iSTRING,t);      srsp++
 #define asCHAR(n)    instrInt(iCHAR,n);         srsp++
 #define asLOAD(n)    instrInt(iLOAD,n);         srsp++
@@ -365,9 +386,21 @@ static Void local asEND() {            /* Fix addresses in assembled code  */
 #define asCELL(c)    instrCell(iCELL,c);        srsp++
 #define asFAIL()     instrNone(iFAIL)
 
+/* Peephole optimisations are unsafe if we're at the start of a new
+   'basic block' (newBasicBlock is TRUE) -- e.g.,
+
+	f a b = if (if a then True else b) then foo else bar
+
+   After having emitted the code for the conditional, we want to evaluate
+   it, BUT we better _not_ look at the last instruction of the code
+   comprising the conditional expression and decide whether or not to
+   peephole the EVAL into an LEVAL.
+*/
+
 static Void local asEVAL() {            /* load and eval stack element     */
-    if (instrAt(lastInstr)==iLOAD)
-	instrAt(lastInstr) = iLEVAL;
+    if (!newBasicBlock && instrAt(lastInstr)==iLOAD)
+					/* Peephole optimisation:          */
+	instrAt(lastInstr) = iLEVAL;	/* LOAD n; EVAL ===> LEVAL n       */
     else
 	instrNone(iEVAL);
     srsp--;
@@ -387,7 +420,8 @@ Label l; {
 
 static Void local asSLIDE(n)            /* Slide results down stack        */
 Int n; {
-    if (instrAt(lastInstr)==iSLIDE)     /* Peephole optimisation:          */
+    if (!newBasicBlock && instrAt(lastInstr)==iSLIDE)
+					/* Peephole optimisation:          */
 	intAt(lastInstr+1)+=n;          /* SLIDE n;SLIDE m ===> SLIDE (n+m)*/
     else
 	instrInt(iSLIDE,n);
@@ -396,7 +430,8 @@ Int n; {
 
 static Void local asMKAP(n)             /* Make application nodes ...      */
 Int n; {
-    if (instrAt(lastInstr)==iMKAP)      /* Peephole optimisation:          */
+    if (!newBasicBlock && instrAt(lastInstr)==iMKAP)
+					/* Peephole optimisation:          */
 	intAt(lastInstr+1)+=n;          /* MKAP n; MKAP m  ===> MKAP (n+m) */
     else
 	instrInt(iMKAP,n);
@@ -405,8 +440,8 @@ Int n; {
 
 static Void local asUPDATE(n)           /* Update node ...                 */
 Int n; {
-    if (instrAt(lastInstr)==iMKAP) {    /* Peephole optimisations:         */
-	Int m = intAt(lastInstr+1);
+    if (!newBasicBlock && instrAt(lastInstr)==iMKAP) {
+	Int m = intAt(lastInstr+1);	/* Peephole optimisations:         */
 	nextInstr(lastInstr);
 	if (m==1)                       /* MKAP 1; UPDATE p ===> UPDAP p   */
 	    instrInt(iUPDAP,n);
@@ -421,8 +456,8 @@ Int n; {
 }
 
 static Void local asRUPDATE() {         /* Update node and return ...      */
-    if (instrAt(lastInstr)==iMKAP) {    /* Peephole optimisations:         */
-	Int m = intAt(lastInstr+1);
+    if (!newBasicBlock && instrAt(lastInstr)==iMKAP) {
+	Int m = intAt(lastInstr+1);	/* Peephole optimisations:         */
 	nextInstr(lastInstr);
 	if (m==1)                       /* MKAP 1; RUPDATE ===> RUPDAP     */
 	    instrNone(iRUPDAP);
@@ -529,7 +564,7 @@ Addr pc; {
 	case iCELL   : pc = dissCell(pc,"CELL");     break;
 	case iCHAR   : pc = dissInt(pc,"CHAR");      break;
 	case iINT    : pc = dissInt(pc,"INT");       break;
-	case iFLOAT  : pc = dissFloat(pc,"FLOAT");   break;
+	case iDOUBLE : pc = dissDouble(pc,"DOUBLE"); break;
 	case iSTRING : pc = dissText(pc,"STRING");   break;
 	case iMKAP   : pc = dissInt(pc,"MKAP");      break;
 	case iUPDATE : pc = dissInt(pc,"UPDATE");    break;
@@ -576,17 +611,12 @@ String s; {
     return pc+2;
 }
 
-static Addr local dissFloat(pc,s)      /* dissassemble instr with Float arg*/
+static Addr local dissDouble(pc,s)      /* dissassemble instr with Double arg*/
 Addr   pc;
 String s; {
-#if BREAK_FLOATS
     Printf("%s\t%s\n",s,
-	floatToString(floatFromParts(cellAt(pc+1),cellAt(pc+2))));
+	doubleToString(doubleFromParts(cellAt(pc+1),cellAt(pc+2))));
     return pc+3;
-#else
-    Printf("%s\t%s\n",s,floatToString((FloatPro)floatAt(pc+1)));
-    return pc+2;
-#endif
 }
 
 static Addr local dissCell(pc,s)       /* dissassemble instr with Cell arg */
@@ -685,7 +715,7 @@ Int  co; {                              /* perform no evaluation           */
 	case INTCELL   : asINTEGER(intOf(e));
 			 break;
 
-	case FLOATCELL : asFLOAT(floatOf(e));
+	case DOUBLECELL : asDOUBLE(doubleOf(e));
 			 break;
 
 	case STRCELL   : asSTRING(textOf(e));
@@ -883,7 +913,7 @@ Label d; {
 			 asGOTO(d);
 			 break;
 
-	case FLOATCELL : asFLOAT(floatOf(e));
+	case DOUBLECELL : asDOUBLE(doubleOf(e));
 			 asGOTO(d);
 			 break;
 
@@ -949,7 +979,7 @@ Label  f, d; {
 		      /* ToDo: should this be the same as fromInt? */
 #endif
 		      else if (h==nameFromDouble) {
-			  asFLOAT(floatOf(arg(discr)));
+			  asDOUBLE(doubleOf(arg(discr)));
 			  make(arg(fun(discr)),co,SHOULDNTFAIL,RUNON);
 			  asCELL(namePmFlt);
 		      }
@@ -1206,13 +1236,9 @@ List scs; {                             /* in incr order of strict comps.  */
 
 #if TREX
 Name nameInsFld;			/* Hooks to Trex library	   */
-Name nameShowRecRow;
+Name nameShowRecRow;                    /* static.c:trexLoad() binds them  */
 Name nameEqRecRow;
 
-Cell trexInsFld     = NIL;
-Cell trexShowRecRow = NIL;
-Cell trexEqRecRow   = NIL;
-
 Name implementRecShw(t,parent)		/* Build implementation for record */
 Text t; 				/* display function.		   */
 Cell parent; {
@@ -1226,8 +1252,6 @@ Cell parent; {
     asRETURN();
     asEND();
     name(n).code    = startInstr;
-    nameInsFld      = findQualName(trexInsFld);
-    nameShowRecRow  = findQualName(trexShowRecRow);
     return n;
 }
 
@@ -1244,8 +1268,6 @@ Cell parent; {
     asRETURN();
     asEND();
     name(n).code   = startInstr;
-    nameInsFld     = findQualName(trexInsFld);
-    nameEqRecRow   = findQualName(trexEqRecRow);
     return n;
 }
 #endif
@@ -1258,6 +1280,7 @@ Int   whnfArgs;                         /* number of arguments of whnf term*/
 Cell  whnfHead;                         /* head cell of term in whnf       */
 Int   whnfInt;                          /* value of INTCELL (in whnf)      */
 Float whnfFloat;                        /* value of FLOATCELL (in whnf)    */
+Double whnfDouble;                      /* value of DOUBLECELL (in whnf)   */
 Long  numReductions;                    /* number of reductions counted    */
 #if PROFILING
 #define saveProducer(n)                 { Name old = producer; producer = n
@@ -1270,7 +1293,7 @@ Long  numReductions;                    /* number of reductions counted    */
 #define restoreProducer()               /* nothing */
 #endif
 
-static Cell    errorRedex;              /* irreducible error expression    */
+static Cell    exception;               /* Exception thrown                */
 static jmp_buf *evalError = 0;          /* jump buffer for eval errors     */
 
 #if GIMME_STACK_DUMPS
@@ -1338,6 +1361,11 @@ unw:switch (whatIs(n)) {                /* unwind spine of application     */
 				 } while (--ar>0);
 
 				 saveProducer(n);
+#ifdef DOTNET
+				 if (name(n).foreignInfo != NIL) 
+				    primInvoker(root,n);
+				 else
+#endif
 				 if (name(n).primDef)       /* reduce      */
 				     (*name(n).primDef)(root);
 				 else
@@ -1353,6 +1381,11 @@ unw:switch (whatIs(n)) {                /* unwind spine of application     */
 				     StackPtr root = sp;
 				     push(n);               /* save CAF    */
 				     saveProducer(n);
+#ifdef DOTNET
+				     if (name(n).foreignInfo != NIL) 
+				       primInvoker(root,n);
+				     else
+#endif
 				     if (name(n).primDef)
 					 (*name(n).primDef)(sp);
 				     else
@@ -1442,6 +1475,11 @@ unw:switch (whatIs(n)) {                /* unwind spine of application     */
                 }
 
                 saveProducer(n);
+#ifdef DOTNET
+		if (name(n).foreignInfo != NIL) 
+		  primInvoker(root,n);
+		else
+#endif
                 if (name(n).primDef){      /* reduce      */
                     (*name(n).primDef)(root);
                 }   
@@ -1468,6 +1506,11 @@ unw:switch (whatIs(n)) {                /* unwind spine of application     */
                     StackPtr root = sp;
                     push(n);               /* save CAF    */
                     saveProducer(n);
+#ifdef DOTNET
+		    if (name(n).foreignInfo != NIL)
+		      primInvoker(root,n);
+		    else
+#endif
                     if (name(n).primDef)
                         (*name(n).primDef)(sp);
                     else
@@ -1504,6 +1547,9 @@ unw:switch (whatIs(n)) {                /* unwind spine of application     */
 	case FLOATCELL : whnfFloat = (Float)floatOf(n);
 			 break;
 
+	case DOUBLECELL : whnfDouble = (Double)doubleOf(n);
+			 break;
+
 	case STRCELL   : evalString(n);
 			 goto unw;
     }
@@ -1591,6 +1637,9 @@ unw:switch (whatIs(n)) {
 	case FLOATCELL : whnfFloat = (Float)floatOf(n);
 			 break;
 
+	case DOUBLECELL : whnfDouble = (Double)doubleOf(n);
+			 break;
+
 	case STRCELL   : evalString(n);
 			 goto unw;
     }
@@ -1600,20 +1649,17 @@ unw:switch (whatIs(n)) {
 static Void local evalString(n)        /* expand STRCELL at node n         */
 Cell n; {
     Text t = textOf(n);
-    Int  c = textToStr(t)[0];
-    if (c==0) {
+    String s = textToStr(t);
+    Char c;
+    if (*s==0) {
 	fst(n) = INDIRECT;
 	snd(n) = nameNil;
 	return;
     }
-    else if (c=='\\') {
-	c = textToStr(++t)[0];
-	if (c!='\\')
-	    c = 0;
-    }
+    c = getStrChr(&s);
     push(n);                           /* protect n during mkStr           */
     fst(n) = consChar(c);
-    snd(n) = mkStr(++t);
+    snd(n) = mkStr(t + (s-textToStr(t)));
     drop();
 }
 
@@ -1666,16 +1712,10 @@ static  void *labs[] = { INSTRLIST };
 			pc++;
 			Continue;
 
-#if BREAK_FLOATS
-	Case(iFLOAT)  : push(mkFloat(floatFromParts      /* load dbl const */
+	Case(iDOUBLE)  : push(mkDouble(doubleFromParts   /* load dbl const */
 				(pc->cell,(pc+1)->cell)));
 			pc+=2;
 			Continue;
-#else
-	Case(iFLOAT)  : push(mkFloat(pc->mfloat));       /* load float cnst*/
-			pc++;
-			Continue;
-#endif
 
 	Case(iSTRING) : push(mkStr(pc->text));           /* load str const */
 			pc++;
@@ -1798,7 +1838,7 @@ static  void *labs[] = { INSTRLIST };
 			Continue;
 
 	Case(iTABLE)  :
-	Case(iFAIL)   : evalFails(root);                 /* cannot reduce  */
+	Case(iFAIL)   : evalFails(root);		 /* cannot reduce  */
 			return;/*NOT REACHED*/
 
     EndDispatch
@@ -1810,8 +1850,8 @@ static  void *labs[] = { INSTRLIST };
 }
 
 Cell evalWithNoError(e)                /* Evaluate expression, returning   */
-Cell e; {                              /* NIL if successful, irreducible   */
-    Cell badRedex;                     /* expression if not...             */
+Cell e; {                              /* NIL if successful,               */
+    Cell caughtEx;                     /* Exception value if not...        */
     jmp_buf *oldCatch = evalError;
 
 #if JMPBUF_ARRAY
@@ -1819,43 +1859,43 @@ Cell e; {                              /* NIL if successful, irreducible   */
     evalError = catcherr;
     if (setjmp(catcherr[0])==0) {
 	eval(e);
-	badRedex = NIL;
+	caughtEx = NIL;
     }
     else
-	badRedex = errorRedex;
+	caughtEx = exception;
 #else
     jmp_buf catcherr;
     evalError = &catcherr;
     if (setjmp(catcherr)==0) {
 	eval(e);
-	badRedex = NIL;
+	caughtEx = NIL;
     }
     else
-	badRedex = errorRedex;
+	caughtEx = exception;
 #endif
     evalError = oldCatch;
-    return badRedex;
+    return caughtEx;
 }
 
 Void evalFails(root)                   /* Eval of current redex fails     */
 StackPtr root; {
-    errorRedex = stack(root);          /* get error & bypass indirections */
+    Cell errorRedex = stack(root);     /* get error & bypass indirections */
     while (isPair(errorRedex) && fst(errorRedex)==INDIRECT)
 	errorRedex = snd(errorRedex);
+    throwException(ap(namePatternMatchFail,
+	ap(ap(ap(nameNPrint, mkInt(MIN_PREC)), errorRedex), nameNil)));
+}
 
+Void throwException(ex)
+Cell ex; {
+    exception = ex;
 #if OBSERVATIONS
     obsCount=0;
 #endif
-#ifdef EMBEDDED
-    longjmp(*evalError,1);
-#else
-    if (failOnError)
-	abandon("Program",errorRedex);
-    else if (evalError)
+    if (evalError)
 	longjmp(*evalError,1);
     else
-	internal("uncaught eval error");
-#endif
+	internal("uncaught exception");
 }
 
 /* --------------------------------------------------------------------------
@@ -1865,38 +1905,40 @@ StackPtr root; {
 Void machine(what)
 Int what; {
     switch (what) {
-	case INSTALL : memory  = (Memory)farCalloc(NUM_ADDRS,sizeof(MemCell));
+	case INSTALL : 
+#if WANT_FIXED_SIZE_TABLES
+	               memory  = (Memory)farCalloc(NUM_ADDRS,sizeof(MemCell));
+#else
+		       /* prelude.h defines NUM_ADDRS; 0 is the upper bound (=> unbounded.) */
+		       dynMemory = allocDynTable(sizeof(Memory), NUM_ADDRS, 0, "memory");
+		       memory    = (Memory)(dynMemory->data);
+#endif
 		       if (memory==0)
 			   fatal("Cannot allocate program memory");
 		       instrNone(iFAIL);
 		       noMatch = lastInstr;
-#if TREX
-		       trexInsFld
-			= mkQVar(findText("Trex"),findText("insertField"));
-		       trexShowRecRow
-			= mkQVar(findText("Trex"),findText("showRecRow"));
-		       trexEqRecRow
-			= mkQVar(findText("Trex"),findText("eqRecRow"));
-#endif
 		       break;
 
-	case MARK    :
-#if TREX
-		       mark(trexInsFld);
-		       mark(trexShowRecRow);
-		       mark(trexEqRecRow);
-#endif
-		       break;
+	case MARK    : break;
 
+	case RESET   : evalError = 0;
 #if GIMME_STACK_DUMPS
-	case RESET   : rootsp = (-1);
-		       break;
+		       rootsp = (-1);
 #endif
+		       break;
 
 #if OBSERVATIONS
 	case BREAK   : obsCount = 0;
 	               break;
 #endif
+
+        case EXIT    : 
+#if WANT_FIXED_SIZE_TABLES
+	               free(memory);
+#else
+		       if (dynMemory) freeDynTable(dynMemory);
+#endif
+	               break;
     }
 }
 
