@@ -3,20 +3,21 @@
  * and for debugging.
  *
  * The Hugs 98 system is Copyright (c) Mark P Jones, Alastair Reid, the
- * Yale Haskell Group, and the Oregon Graduate Institute of Science and
- * Technology, 1994-1999, All rights reserved.  It is distributed as
- * free software under the license in the file "License", which is
- * included in the distribution.
+ * Yale Haskell Group, and the OGI School of Science & Engineering at OHSU,
+ * 1994-2003, All rights reserved.  It is distributed as free software under
+ * the license in the file "License", which is included in the distribution.
  *
  * $RCSfile: output.c,v $
- * $Revision: 1.16 $
- * $Date: 2001/02/14 12:15:05 $
+ * $Revision: 1.38 $
+ * $Date: 2004/01/11 00:34:30 $
  * ------------------------------------------------------------------------*/
 
 #include "prelude.h"
 #include "storage.h"
 #include "connect.h"
 #include "errors.h"
+#include "output.h"
+#include "char.h"
 #include <ctype.h>
 
 #if OBSERVATIONS
@@ -46,8 +47,9 @@ static Void local putSimpleAp    Args((Cell,Int));
 static Void local putTuple       Args((Int,Cell));
 static Int  local unusedTups     Args((Int,Cell));
 static Void local unlexVar       Args((Text));
+static Void local unlexFullVar   Args((Name));
 static Void local unlexOp        Args((Text));
-static Void local unlexCharConst Args((Cell));
+static Void local unlexCharConst Args((Char));
 static Void local unlexStrConst  Args((Text));
 
 static Void local putSigType     Args((Cell));
@@ -65,10 +67,13 @@ static Void local putKinds	 Args((Kinds));
 #if OBSERVATIONS
 static Bool local printObsList   Args((Cell,Int,Bool));
 static Void local printArg       Args((FILE *,Cell));
-static Bool local isCharCell	 Args((Cell));
-static Bool local getCellChar    Args((Cell));
 #endif
 
+static Bool local isStrConst	 Args((Cell));
+static Void local putStrConst	 Args((Cell));
+static Bool local isCharCell	 Args((Cell));
+static Char local getCellChar    Args((Cell));
+
 /* --------------------------------------------------------------------------
  * Basic output routines:
  * ------------------------------------------------------------------------*/
@@ -170,7 +175,7 @@ Cell e; {
 			  putChr(']');
 			  break;
 
-	case AP         : putAp(d,e);
+        case AP         : putAp(d,e);
 			  break;
 
 	case NAME       : 
@@ -257,9 +262,9 @@ Cell e; {
 			  break;
 #endif
 
-	case FLOATCELL  : {   Float f = floatOf(e);
+	case DOUBLECELL : {   Double f = (Double)doubleOf(e);
 			      if (f<0 && d>=UMINUS_PREC) putChr('(');
-			      putStr(floatToString(f));
+			      putStr(doubleToString(f));
 			      if (f<0 && d>=UMINUS_PREC) putChr(')');
 			  }
 			  break;
@@ -367,11 +372,13 @@ Cell e; {
     putDepth--;
 }
 
+
+
 static Void local putFlds(exp,fs)       /* Output exp using labelled fields*/
 Cell exp;
 List fs; {
     put(ALWAYS,exp);
-    putChr('{');
+    putStr(" {");
     for (; nonNull(fs); fs=tl(fs)) {
 	Cell v = hd(fs);
 	if (isVar(v))
@@ -454,9 +461,9 @@ Cell e; {                               /* ignoring dict aps               */
 static Void local putAp(d,e)            /* print application (args>=1)     */
 Int  d;
 Cell e; {
-    Cell   h;
-    Text   t;
-    Syntax sy;
+    Cell   h    = NIL;
+    Text   t    = NIL;
+    Syntax sy   = NIL;
     Int    args = 0;
 
     for (h=e; isAp(h); h=fun(h))        /* find head of expression, looking*/
@@ -486,7 +493,7 @@ Cell e; {
 	case NAME       : if (args==1 &&
 			      ((h==nameFromInt     && isInt(arg(e)))    ||
 			       (h==nameFromInteger && isBignum(arg(e))) ||
-			       (h==nameFromDouble  && isFloat(arg(e))))) {
+			       (h==nameFromDouble  && isDouble(arg(e))))) {
 			      put(d,arg(e));
 			      return;
 			  }
@@ -508,7 +515,7 @@ Cell e; {
 				  putStr(punc);
 				  punc = ", ";
 				  putStr(textToStr(extText(h)));
-				  putStr("=");
+				  putStr(" = ");
 				  put(NEVER,extField(e));
 				  args = 0;
 				  e    = extRow(e);
@@ -549,6 +556,8 @@ Cell e; {
 	unlexOp(t);
 	putChr(')');
     }
+    else if (args==2 && isStrConst(e))
+        putStrConst(e);
     else if (args==2)                  /* infix expr of the form e1 + e2   */
 	putInfix(d,t,sy,arg(maySkipDict(fun(e))),arg(e));
     else {                             /* o/w (e1 + e2) e3 ... en   (n>=3) */
@@ -558,7 +567,28 @@ Cell e; {
     }
 }
 
-#if OBSERVATIONS
+static Void local putStrConst(e)
+Cell e; {
+    putChr('"'); 
+    while (isAp(e) && isAp(fun(e)) && fun(fun(e))==nameCons) {
+        putStr(unlexChar(getCellChar(arg(fun(e))),'"'));
+        e = arg(e);
+    }
+    if (e!=nameNil)
+        internal("putStrConst");
+    putChr('"');
+}
+
+static Bool local isStrConst(e)
+Cell e; {
+    while (isAp(e) && isAp(fun(e)) && fun(fun(e))==nameCons) {
+        if (!isCharCell(arg(fun(e))))
+            return FALSE;
+        e = arg(e);
+    }
+    return e==nameNil;
+}
+
 static Bool local isCharCell(e)
 Cell e; {
     while(1)
@@ -566,25 +596,28 @@ Cell e; {
 	    case CHARCELL	: return 1;
 	    case INDIRECT	: e = snd(e);
 	    			  break;
+#if OBSERVATIONS
 	    case OBSERVE	: e = markedExpr(e);
 	    			  break;
+#endif
 	    default		: return 0;
 	}
 }
 
-static Bool local getCellChar(e)
+static Char local getCellChar(e)
 Cell e; {
     while(1)
         switch(whatIs(e)) {
 	    case CHARCELL	: return charOf(e);
 	    case INDIRECT	: e = snd(e);
 	    			  break;
+#if OBSERVATIONS
 	    case OBSERVE	: e = markedExpr(e);
 	    			  break;
-	    default		: ERRMSG(0) "error in getCellChar" EEND;
+#endif
+	    default		: internal("error in getCellChar");
 	}
 }
-#endif
 
 static Void local putOverInfix(args,t,sy,e)
 Int    args;                           /* infix applied to >= 3 arguments  */
@@ -607,34 +640,13 @@ Syntax sy;                              /* with name t, syntax s           */
 Cell   e, f; {                          /* Left and right operands         */
     Syntax a = assocOf(sy);
     Int    p = precOf(sy);
-
-#if OBSERVATIONS
-    Bool charString = strcmp(textToStr(t),":")==0 && isCharCell(e);
-    if (charString){
-        /* special processing for character lists 			   */
-        if (!inString) { putChr('"'); 
-	                 inString=1;
-		       }
-	putStr(unlexChar(getCellChar(e),'"'));
-    }
-    else {
-#endif
-        OPEN(d>p);
-        put((a==LEFT_ASS ? p : 1+p), e);
-        putChr(' ');
-        unlexOp(t);
-        putChr(' ');
-#if OBSERVATIONS
-    }
-#endif
+    OPEN(d>p);
+    put((a==LEFT_ASS ? p : 1+p), e);
+    putChr(' ');
+    unlexOp(t);
+    putChr(' ');
     put((a==RIGHT_ASS ? p : 1+p), f);
-#if OBSERVATIONS
-    if (inString) { putChr('"'); 
-                    inString=0;
-		  }
-    if (!charString)
-#endif
-        CLOSE(d>p);
+    CLOSE(d>p);
 }
 
 static Void local putSimpleAp(e,n)      /* print application e0 e1 ... en  */
@@ -680,7 +692,6 @@ Cell e; {                               /* args not yet printed ...        */
 static Void local unlexVar(t)          /* print text as a variable name    */
 Text t; {                              /* operator symbols must be enclosed*/
     String s = textToStr(t);           /* in parentheses... except [] ...  */
-
     if ((isascii(s[0]) && isalpha(s[0])) || s[0]=='_' || s[0]=='[' || s[0]=='(')
 	putStr(s);
     else {
@@ -690,6 +701,30 @@ Text t; {                              /* operator symbols must be enclosed*/
     }
 }
 
+static Void local unlexFullVar(n)  /* print text as a variable name    */
+Name n; {                              
+  Module m = name(n).mod;
+  Text t = name(n).text;
+  if (name(n).parent 
+      && isName(name(n).parent)
+      && isPair(name(name(n).parent).defn)
+      && snd(name(name(n).parent).defn) == n) {
+    /* Constructor with strict fields are handled in
+     * a strange manner. Here we print the true construtor name
+     */
+    unlexFullVar(name(n).parent);  
+  } else {
+    if (name(n).primDef) {
+      putStr("Prelude");
+    } else {
+      putStr(textToStr(module(m).text));
+    }
+    putChr('.');
+    putStr(textToStr(t));
+  }
+}
+
+
 static Void local unlexOp(t)           /* print text as operator name      */
 Text t; {                              /* alpha numeric symbols must be    */
     String s = textToStr(t);           /* enclosed by backquotes           */
@@ -704,7 +739,7 @@ Text t; {                              /* alpha numeric symbols must be    */
 }
 
 static Void local unlexCharConst(c)
-Cell c; {
+Char c; {
     putChr('\'');
     putStr(unlexChar(c,'\''));
     putChr('\'');
@@ -719,16 +754,19 @@ Text t; {
     Bool   lastWasEsc   = FALSE;
 
     putChr('\"');
-    for (; *s; s++) {
-	String ch = unlexChar(*s,'\"');
+    while (*s) {
+      String ch;
 	Char   c  = ' ';
+	Char   sc = getStrChr(&s);
+
+	ch = unlexChar(sc,'\"');
 
 	if ((lastWasSO && *ch=='H') ||
 		(lastWasEsc && lastWasDigit && isascii(*ch) && isdigit(*ch)))
 	    putStr("\\&");
 
 	lastWasEsc   = (*ch=='\\');
-	lastWasSO    = (*s==SO);
+	lastWasSO    = (sc==SO);
 	for (; *ch; c = *ch++)
 	    putChr(*ch);
 	lastWasDigit = (isascii(c) && isdigit(c));
@@ -773,7 +811,8 @@ Int  co; {
 	case OFFSET     : pPutOffset(offsetOf(e));
 			  break;
 
-	case NAME       : unlexVar(name(e).text);
+        case NAME       : 
+                          unlexFullVar(e);
 			  break;
 
 #if TREX
@@ -797,9 +836,9 @@ Int  co; {
 			  }
 			  break;
 
-	case FLOATCELL  : {   Float f = floatOf(e);
+	case DOUBLECELL : {   Double f = (Double)doubleOf(e);
 			      if (f<0 && d>=UMINUS_PREC) putChr('(');
-			      putStr(floatToString(f));
+			      putStr(doubleToString(f));
 			      if (f<0 && d>=UMINUS_PREC) putChr(')');
 			  }
 			  break;
@@ -821,17 +860,24 @@ Int  co; {
 			  CLOSE(d>WHERE_PREC);
 			  break;
 
-	case COND       : OPEN(d>COND_PREC);
-			  putStr("if ");
-			  pPut(COND_PREC+1,fst3(snd(e)),co);
-			  putStr(" then ");
-			  pPut(COND_PREC+1,snd3(snd(e)),co);
-			  putStr(" else ");
-			  pPut(COND_PREC+1,thd3(snd(e)),co);
-			  CLOSE(d>COND_PREC);
-			  break;
-
-	default         : internal("pPut");
+        case COND       : { Int  left = outColumn;
+	                    OPEN(d>COND_PREC);
+			    putStr("case ");
+			    pPut(COND_PREC+1,fst3(snd(e)),co);
+			    putStr(" of");
+			    pIndent(left+2);
+			    putStr(" { Prelude.True -> ");
+			    pPut(COND_PREC+1,snd3(snd(e)),co);
+			    pIndent(left+2);
+			    putStr(" ; Prelude.False ->  ");
+			    pPut(COND_PREC+1,thd3(snd(e)),co);
+			    pIndent(left+2);
+			    putStr("}");
+			    CLOSE(d>COND_PREC);
+			    break;
+	                   }
+	default         : printf("[e = %d, whatIs(e) = %d]\n",e,whatIs(e));
+	                  internal("pPut");
     }
 }
 
@@ -869,12 +915,14 @@ Int  ts;                                /* for possibility of either too   */
 Cell e;                                 /* few or too many args to constr  */
 Int  co; {
     Int i;
-    putChr('(');
+    putStr("(#");
+    putInt(ts);
+    putChr(' ');
     if ((i=punusedTups(ts,e,co))>0) {
 	while (--i>0)
-	    putChr(',');
-	putChr(')');
+	    putChr(' ');
     }
+    putChr(')');
 }
 
 static Int local punusedTups(ts,e,co)   /* print first part of tuple expr  */
@@ -883,8 +931,8 @@ Cell e;                                 /* args not yet printed ...        */
 Int  co; {
     if (isAp(e)) {
 	if ((ts=punusedTups(ts,fun(e),co)-1)>=0) {
-	    pPut(NEVER,arg(e),co);
-	    putChr(ts>0?',':')');
+	    pPut(ALWAYS,arg(e),co);
+	    putStr(ts>0?" ":"");
 	}
 	else {
 	    putChr(' ');
@@ -896,7 +944,7 @@ Int  co; {
 
 static Void local pPutOffset(n)         /* pretty print offset number      */
 Int n; {
-    putChr('o');
+    putChr('_');
     putInt(n);
 }
 
@@ -907,17 +955,17 @@ Int  co; {
     Int n    = length(vs);
     Int i;
 
-    putStr("let { ");
+    putStr("let ");
     for (i=0; i<n; i++) {
 	pPutOffset(co+i+1);
 	putChr(' ');
 	pLiftedStart(hd(vs),co+n,"=");
 	vs = tl(vs);
 	if (nonNull(vs))
-	    pIndent(left+6);
+	    pIndent(left+4);
     }
     pIndent(left);
-    putStr("} in  ");
+    putStr("in ");
     return n;
 }
 
@@ -925,10 +973,8 @@ static Void local pLiftedStart(e,co,eq) /* print start of definition       */
 Cell   e;
 Int    co;
 String eq; {
-    if (whatIs(e)!=GUARDED) {
-	putStr(eq);
-	putChr(' ');
-    }
+    putStr(eq);
+    putChr(' ');
     pLifted(e,co,eq);
 }
 
@@ -938,23 +984,32 @@ Int    co;
 String eq; {
     switch (whatIs(e)) {
 	case GUARDED : {   Int  left = outColumn;
+	                   Int count = 0;
+                           Int tmp;
 			   List gs   = snd(e);
 			   if (isNull(gs))
 			       internal("pLifted");
 			   for (;;) {
-			       putStr("| ");
+			       count++;
+			       putStr("case ");
 			       pPut(NEVER,fst(hd(gs)),co);
-			       putChr(' ');
-			       putStr(eq);
-			       putChr(' ');
+			       putStr(" of\n");
+			       pIndent(left + 2);
+			       putStr("{ Prelude.True -> ");
 			       pPut(NEVER,snd(hd(gs)),co);
-			       putStr(";\n");
+			       putStr("\n");
+			       pIndent(left + 2);
+			       putStr("; _ -> \n");
 			       gs = tl(gs);
-			       if (nonNull(gs))
-				   pIndent(left);
-			       else
+			       pIndent(left);
+			       if (!nonNull(gs))
 				   break;
 			   }
+			   putStr("_fatbar ");
+			   for(tmp = 0;tmp < count;tmp++) {
+			     putChr('}');
+			   }
+			   putStr(";\n");
 		       }
 		       break;
 
@@ -963,11 +1018,13 @@ String eq; {
 		       break;
 
 	case FATBAR  : {   Int left = outColumn;
-			   pLifted(fst(snd(e)),co,eq);
-			   pIndent(left);
-			   putStr("FATBAR\n");
-			   pIndent(left);
+                           putStr("let_ _fatbar = \n");
+			   pIndent(left+2);
 			   pLifted(snd(snd(e)),co,eq);
+			   pIndent(left);
+                           putStr("in\n");
+			   pIndent(left+2);
+			   pLifted(fst(snd(e)),co,eq);
 		       }
 		       break;
 
@@ -983,8 +1040,10 @@ String eq; {
 			       putChr(' ');
 			       pLiftedStart(snd(hd(cs)),co+arity,"->");
 			   }
+			   pIndent(left+2);
+			   putStr("_ -> _fatbar\n");
 			   pIndent(left);
-			   putStr("}\n");
+			   putStr("};\n");
 		       }
 		       break;
 
@@ -993,16 +1052,49 @@ String eq; {
 #endif
 	case NUMCASE : {   Int  left = outColumn;
 			   Cell t    = snd(e);
-			   Int  ar;
+			   Cell h     = getHead(snd3(t));
+			   String eqInt     = "Prelude.primPmInt";
+			   String eqInteger = "Prelude.primPmInteger";
+			   String eqDouble  = "Prelude.primPmFlt";
+			   String theEq     = "** BAD EQUALITY **";
+			   Int  ar = 0;
 			   putStr("case ");
+			   switch (whatIs(h)) {
+			   case NAME: 
+			     if (h==nameFromInt) {
+			       theEq = eqInt;
+			     } else if (h == nameFromInteger) {
+			       theEq = eqInteger;
+			     } else if (h == nameFromDouble) {
+			       theEq = eqDouble;
+			     } else {
+			       ERRMSG(0) "error in NUMCASE" EEND;
+			     }
+			     break;
+			   case ADDPAT:
+			       ERRMSG(0) "error in NUMCASE " EEND;
+			     break;
+			   default:
+			       ERRMSG(0) "error in NUMCASE" EEND;
+			     break;			     
+			   }
+			   putStr(theEq);
+			   putStr(" (");
+			   pPut(NEVER,arg(fun(snd3(t))),co);
+			   putStr(") (");
+			   pPut(NEVER,arg(snd3(t)),co);
+			   putStr(") (");
 			   pPut(NEVER,fst3(t),co);
-			   putStr(" of {\n");
+			   putStr(") of\n");
 			   pIndent(left+2);
-			   ar =  pDiscr(snd3(t),co);
-			   putChr(' ');
+			   putStr("{ Prelude.True ");
 			   pLiftedStart(thd3(t),co+ar,"->");
+			   putStr("\n");
+			   pIndent(left+2);
+			   putStr("  _ -> _fatbar\n");
+			   pIndent(left+2);
+			   putStr("};\n");
 			   pIndent(left);
-			   putStr("}\n");
 		       }
 		       break;
 
@@ -1028,7 +1120,7 @@ Int  co; {
 
 	case NAME     : {   Int i = 0;
 			    arity = name(d).arity;
-			    unlexVar(name(d).text);
+			    unlexFullVar(d);
 			    for (; i<arity; ++i) {
 				putChr(' ');
 				pPutOffset(co+arity-i);
@@ -1038,13 +1130,14 @@ Int  co; {
 
 	case TUPLE    : {   Int i = 0;
 			    arity = tupleOf(d);
-			    putChr('(');
+			    putChr('#');
+			    putInt(arity);
+			    putChr(' ');
 			    pPutOffset(co+arity);
 			    while (++i<arity) {
-				putChr(',');
+				putChr(' ');
 				pPutOffset(co+arity-i);
 			    }
-			    putChr(')');
 			}
 			break;
 
@@ -1070,6 +1163,7 @@ Int   arity;
 Cell  e; {
     Int i;
     outputStream = fp;
+
     putChr('\n');
     outColumn = 0;
     unlexVar(t);
@@ -1351,8 +1445,10 @@ Kind k; {
 		       }
 		       else
 			   putKind(fst(k));
-		       putStr(" -> ");
-		       putKind(snd(k));
+		       if (whatIs(snd(k)) != NIL) {
+			   putStr(" -> ");
+			   putKind(snd(k));
+		       }
 		       break;
 
 #if TREX
@@ -1414,15 +1510,17 @@ Cell header;
     Int seq, n=0;
     Cell j;
 
-    for (j=firstObs(header); j!=header; j=nextObs(j)){
+    for (j=firstObs(header); j!=header; j=nextObs(j)) {
         seq  = intOf(seqObs(j));
-        if (seq < 0)            /* non-functional value            */
+        if (seq < 0) {          /* non-functional value            */
             n++;    
-        else if (seq!=0)                /* a function observation  */
-            if (whatIs(exprObs(j)) == OBSERVEHEAD)
-                n += countObsList(exprObs(j));
-            else
-                n++;
+        } else if (seq!=0) {    /* a function observation  */
+	   if (whatIs(exprObs(j)) == OBSERVEHEAD) {
+	       n += countObsList(exprObs(j));
+           } else {
+	       n++;
+	   }
+	}
     }
     return n;
 }
@@ -1477,52 +1575,49 @@ Cell header;
 Int  indent;
 Bool funPrint; 
 {
-    Cell j, resultExp;
-    Int seq, appN, argN, i;
+    Cell j, resultExp = 0;
+    Int seq=0, appN, argN;
     Bool firstApp = 1;
 
-	    for (j=firstObs(header); j!=header; j=nextObs(j)){
-	        seq  = intOf(seqObs(j));
-		appN = seqNum(seq);
-		argN = argNum(seq);
+    for (j=firstObs(header); j!=header; j=nextObs(j)){
+        seq  = intOf(seqObs(j));
+        appN = seqNum(seq);
+        argN = argNum(seq);
 
-		if (seq < 0){		/* non-functional value		   */
-		     printArg(stdout, exprObs(j));
-		     if (!funPrint) newLine(indent);
-		}
-		else if (seq!=0) {		/* a function observation  */
-		    funPrint =1;
-		    if (argN == 0){		/* the result expr	   */
-		        if (firstApp){ 		/* print previous result   */
-			    firstApp = 0;
-			    indent   = outColumn;
-			    putStr("{ ");
-			}
-			else { 
-			    putStr(" -> ");
-			    printExp(stdout,resultExp);
-			    newLine(indent); 
-			    putStr(", ");
-			}
-		        resultExp = exprObs(j);
-			putStr ("\\ ");
-		    }
-		    else {			/* an arg expr		   */
-			if (whatIs(exprObs(j)) == OBSERVEHEAD)
-			    printObsList(exprObs(j), indent, TRUE);
-			else
-			    printArg(stdout,exprObs(j));
-			putStr(" ");
-		    }
-		}
-	    }
-	    if (seq >= 0){ 	/* print result of last fun. obs. in list  */
-	        putStr(" -> ");
-		printExp(stdout,resultExp);
-		newLine(indent);
-		putStr("}");
-	    }
-	    return(funPrint);
+        if ( seq < 0 ) {         /* non-functional value        */
+            printArg(stdout, exprObs(j));
+            if (!funPrint) newLine(indent);
+        } else if (seq!=0) {     /* a function observation  */
+            funPrint =1;
+            if (argN == 0){      /* the result expr     */
+                if (firstApp){   /* print previous result   */
+                    firstApp = 0;
+                    indent   = outColumn;
+                    putStr("{ ");
+                } else { 
+                    putStr(" -> ");
+                    printExp(stdout,resultExp);
+                    newLine(indent); 
+                    putStr(", ");
+                }
+                resultExp = exprObs(j);
+                putStr ("\\ ");
+            } else {          /* an arg expr         */
+                if (whatIs(exprObs(j)) == OBSERVEHEAD)
+                    printObsList(exprObs(j), indent, TRUE);
+                else
+                    printArg(stdout,exprObs(j));
+                putStr(" ");
+            }
+        }
+    }
+    if (seq >= 0){  /* print result of last fun. obs. in list  */
+        putStr(" -> ");
+        printExp(stdout,resultExp);
+        newLine(indent);
+        putStr("}");
+    }
+    return(funPrint);
 }
 #endif
 
@@ -1615,4 +1710,5 @@ Pair  fd; {
     }
 }
 
+
 /*-------------------------------------------------------------------------*/
