@@ -3,14 +3,13 @@
  * used for certain primitive types.
  *
  * The Hugs 98 system is Copyright (c) Mark P Jones, Alastair Reid, the
- * Yale Haskell Group, and the Oregon Graduate Institute of Science and
- * Technology, 1994-1999, All rights reserved.  It is distributed as
- * free software under the license in the file "License", which is
- * included in the distribution.
+ * Yale Haskell Group, and the OGI School of Science & Engineering at OHSU,
+ * 1994-2003, All rights reserved.  It is distributed as free software under
+ * the license in the file "License", which is included in the distribution.
  *
  * $RCSfile: printer.c,v $
- * $Revision: 1.4 $
- * $Date: 2001/01/02 18:21:41 $
+ * $Revision: 1.14 $
+ * $Date: 2003/12/02 12:15:53 $
  * ------------------------------------------------------------------------*/
 
 static Void   local printer		Args((Name,Int));
@@ -20,9 +19,10 @@ static Void   local outOp		Args((Name));
 static Void   local outStr		Args((String));
 static Void   local outPr		Args((Name,Int,Cell));
 static Void   local outLPr		Args((Name,Cell));
+static Void   local outException	Args((Cell));
 static Void   local outBadRedex		Args((Cell));
-static Cell   local printDBadRedex	Args((Cell,Cell));
-static Cell   local printBadRedex	Args((Cell,Cell));
+static Cell   local printDException	Args((Cell));
+static Cell   local printException	Args((Cell,Cell));
 
 static Name nameLPrint, nameNLPrint;	/* list printing primitives	   */
 static Name nameSPrint, nameNSPrint;	/* string printing primitives	   */
@@ -77,7 +77,7 @@ static struct primitive printerPrimTable[] = {
   {"sprint",		2, primSPrint},
   {"nsprint",		2, primNSPrint},
   {"primShowsInt",	3, primBPrint},
-  {"primShowsAddr",	3, primBPrint},
+  {"primShowsPtr",	3, primBPrint},
   {"primShowsInteger",	3, primBPrint},
   {"primShowsFloat",	3, primBPrint},
   {"primShowsDouble",	3, primBPrint},
@@ -97,7 +97,7 @@ primFun(primPrint) {			/* Evaluate and print term	   */
     out  = NIL;
     if (nonNull(temp)) {
 	push(temp);
-	outBadRedex(top());
+	outException(top());
     }
     else
 	printer(namePrint,d);
@@ -146,7 +146,7 @@ Int  d; {				/* precedence level		   */
 				    if (nonNull(temp)) {
 					push(temp);
 					outCh('[');
-					outBadRedex(top());
+					outException(top());
 					sp = ksp;
 					outLPr(nameLPrint,pushed(1));
 				    }
@@ -238,7 +238,14 @@ Int  d; {				/* precedence level		   */
 
 	case PTRCELL  : {   Pointer p = ptrOf(whnfHead);
 			    char buffer[32];
+			    char spec[16];
+			    /* Fall into line with how GHC shows Addrs  */
+			    sprintf(spec,"0x%%.%dx", (SIZEOF_INTP)*2);
+			    sprintf(buffer,spec,(long)p);
+#if 0
+			    /* Old skool */
 			    sprintf(buffer,"%p",p);
+#endif
 			    outStr(buffer);
 			    pr  = nameNPrint;
 			}
@@ -277,24 +284,30 @@ Int  d; {				/* precedence level		   */
 			pr = nameNPrint;
 			break;
 
+	case DOUBLECELL:if (whnfDouble<0.0 && d>=UMINUS_PREC)
+			    outCh('(');
+			outStr(doubleToString(whnfDouble));
+			if (whnfDouble<0.0 && d>=UMINUS_PREC)
+			    outCh(')');
+			pr = nameNPrint;
+			break;
+
 #if HASKELL_ARRAYS
 	case ARRAY    : outStr("{array}");
 			pr = nameNPrint;
 			break;
 #endif
 
-#if LAZY_ST
+#if IO_REFS
 	case MUTVAR   : outStr("{mutable variable}");
 			pr = nameNPrint;
 			break;
 #endif
 
-#if INTERNAL_PRIMS
 	case HUGSOBJECT: 
 			outStr("{Cell ...}");
 			pr = nameNPrint;
 			break;
-#endif
 
 #if TREX
 	case RECORD   : outStr("{record}");
@@ -340,7 +353,7 @@ primFun(primLPrint) {			/* evaluate and print list	   */
     if (nonNull(temp)) {
 	push(temp);
 	outStr("] ++ ");
-	outBadRedex(top());
+	outException(top());
     }
     else if (whnfHead==nameCons && whnfArgs==2) {
 	outCh(',');
@@ -379,7 +392,7 @@ primFun(primSPrint) {			/* evaluate and print string	   */
     if (nonNull(temp)) {
 	push(temp);
 	outStr("\" ++ ");
-	outBadRedex(top());
+	outException(top());
     }
     else if (whnfHead==nameCons && whnfArgs==2) {
 	temp = evalWithNoError(top());	/* primArg(4), primArg(3) contain  */
@@ -387,7 +400,7 @@ primFun(primSPrint) {			/* evaluate and print string	   */
 	if (nonNull(temp)) {
 	    push(temp);
 	    outStr("\" ++ [");
-	    outBadRedex(top());
+	    outException(top());
 	    outLPr(nameLPrint,primArg(3));
 	}
 	else if (isChar(whnfHead) && whnfArgs==0) {
@@ -488,7 +501,7 @@ Name nm; {
 static Void local outStr(s)		/* output string s		   */
 String s; {
     while (*s)
-	outCh(*s++);
+	outCh(ExtractChar(s));
 }
 
 static Void local outPr(pr,d,e)		/* output expr e with printer pr,  */
@@ -507,6 +520,19 @@ Cell xs; {
     fst(out) = ap(pr,xs);
 }
 
+static Void local outException(ex)	/* Produce expr to print exception */
+Cell ex; {
+    outCh('{');
+    if (isAp(ex) && fun(ex)==nameErrorCall) {
+	outStr("error ");
+	outPr(nameNPrint,FUN_PREC,arg(ex));
+    } else {
+	outStr("throw ");
+	outPr(nameNPrint,FUN_PREC,ex);
+    }
+    outCh('}');
+}
+
 static Void local outBadRedex(rx)	/* Produce expr to print bad redex */
 Cell rx; {
     outCh('{');
@@ -514,25 +540,25 @@ Cell rx; {
     outCh('}');
 }
 
-static Cell local printDBadRedex(rx,rs)	/* Produce expression for bad	   */
-Cell rx, rs; {				/* redex with special handling	   */
-    if (isAp(rx) && fun(rx)==nameError) /* of {error str} redexes	   */
-	return arg(rx);
+static Cell local printDException(ex)	/* Produce expression for exception*/
+Cell ex; {				/* with special handling	   */
+    if (isAp(ex) && fun(ex)==nameErrorCall)/* of {error str} exceptions    */
+	return arg(ex);
     else
-	return printBadRedex(rx,rs);
+	return printException(ex,nameNil);
 }
 
-static Cell local printBadRedex(rx,rs)	/* produce expression for bad	   */
-Cell rx, rs; {				/* redex			   */
+static Cell local printException(ex,rs)	/* produce expression for exception*/
+Cell ex, rs; {
    out = NIL;
-   outBadRedex(rx);
+   outException(ex);
    return revOnto(out,rs);
 }
 
-Void abandon(what,rx)			/* abandon computation		   */
+Void abandon(what,ex)			/* abandon computation		   */
 String what;
-Cell   rx; {
-    push(printDBadRedex(rx,nameNil));
+Cell   ex; {
+    push(printDException(ex));
     out   = NIL;
     outCh('\n');
     outStr(what);
