@@ -2,31 +2,34 @@
  * Standalone hugs system
  *
  * The Hugs 98 system is Copyright (c) Mark P Jones, Alastair Reid, the
- * Yale Haskell Group, and the Oregon Graduate Institute of Science and
- * Technology, 1994-1999, All rights reserved.  It is distributed as
- * free software under the license in the file "License", which is
- * included in the distribution.
+ * Yale Haskell Group, and the OGI School of Science & Engineering at OHSU,
+ * 1994-2003, All rights reserved.  It is distributed as free software under
+ * the license in the file "License", which is included in the distribution.
  *
  * $RCSfile: runhugs.c,v $
- * $Revision: 1.3 $
- * $Date: 1999/09/13 11:01:06 $
+ * $Revision: 1.22 $
+ * $Date: 2004/12/01 17:33:18 $
  * ------------------------------------------------------------------------*/
 
 #include <stdio.h>
 #include <stdlib.h>
-#include "config.h"
-#include "options.h"
+#define HUGS_SERVER 1
+
+#include "prelude.h"
+#include "storage.h"
+#include "connect.h"
+#include "machdep.h"
+#include "observe.h"
 #include "server.h"
 
-#if HAVE_CONSOLE_H
-# include <console.h>
+#if defined(_MSC_VER) && !defined(_MANAGED)
+#include <windows.h>
 #endif
 
 extern int  main      Args((int, char**));
 static void check     Args((void));
 static void loadHugs  Args((int, char**));
 
-extern HugsServerAPI* initHugsServer Args((int,char**));
 static HugsServerAPI* hugs = 0;
 
 static void check() {
@@ -43,7 +46,7 @@ int    argc;
 char* argv[]; {
     hugs = initHugsServer(argc,argv);
     if (NULL == hugs) {
-	fprintf(stderr,"runhugs: Unable to initialise Hugs\n");
+	fprintf(stderr,"runhugs: Unable to initialise Hugs (%s)\n", lastError);
 	fflush(stderr);
 	exit(1);
     }
@@ -58,34 +61,25 @@ char* argv[]; {
 int main(argc,argv)
 int    argc;
 char* argv[]; {
-    int    exitCode;
+    int    exitCode = 0;
     char** hugs_argv;
     int    hugs_argc;
+    char*  progname;
 
-#ifdef HAVE_CONSOLE_H /* Macintosh port */
-    _ftype = 'TEXT';
-    _fcreator = 'R*ch';		/* // 'KAHL';	//'*TEX';	//'ttxt'; */
-
-    console_options.top = 40;
-    console_options.left = 6;
-
-    /* Example of combinations (nrows, ncols, txFont, txSize):
-	(35, 100, 22, 10)
-	(38, 120, 22, 9)
-    */
-    console_options.nrows = 34;
-    console_options.ncols = 80;
-
-    console_options.pause_atexit = 1;
-    console_options.title = "\pHugs" HUGS_VERSION;
-
-    console_options.txFont = 22;	/* 22 = Courier			   */
-    console_options.txSize = 10;
-
-    /* console_options.procID = 5; */
+    progname = argv ? argv[0] : "runhugs";
+    if (!initSystem()) {
+      fprintf(stderr,"%s: failed to initialize, exiting\n", progname);
+      fflush(stderr);
+      exit(1);
+    }
+#if __MWERKS__ && macintosh
     argc = ccommand(&argv);
 #endif
 
+#if defined(FFI_COMPILER)
+    generateFFI = TRUE;
+#endif
+
     /* skip over any option flags before the program name */
     {
 	int i = 1; /* ignore first arg - name of this program */
@@ -103,22 +97,46 @@ char* argv[]; {
     }
 
     if (argc < 1) {
-	fprintf(stderr,"runhugs: missing file argument\n");
+	fprintf(stderr,"%s: missing file argument\n",progname);
 	fflush(stderr);
 	exit(1);
     }
 
+#if defined(_MSC_VER) && !defined(_MANAGED)
+    __try {
+#endif
+
     loadHugs(hugs_argc, hugs_argv);
 
+#if defined(FFI_COMPILER)
+    /* all arguments following the module name are passed to the C compiler */
+    {
+	int i;
+	for (i=1; i<argc; ++i)
+	    ffiSetFlags(argv[i]);
+    }
+
+    hugs->loadFile(argv[0]);
+    check();
+#else
+    /* all arguments following the module name are available via getArgs */
     hugs->loadFile(argv[0]);
     check();
 
     hugs->setHugsArgs(argc,argv);
-
-    hugs->lookupName("Main","main");
+    hugs->pushHVal(hugs->compileExpr("Main","main >> return () :: IO ()"));
     exitCode = hugs->doIO();
     check();
+#endif
 
+#if defined(_MSC_VER) && !defined(_MANAGED)
+    } __except ( ((GetExceptionCode() == EXCEPTION_STACK_OVERFLOW) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH) ) {
+      fatal("C stack overflow");
+    }
+#endif
+
+    shutdownHugsServer(hugs);
+    
     exit(exitCode);
     return 0;/*NOTUSED*/
 }
