@@ -2,16 +2,15 @@
  * WinBrows.c:	Jos Enrique Gallardo Ruiz, Feb 1999
  *              With modifications by mpj/adr for Hugs, 1995-97 
  *
- * Hugs for Windows is Copyright (c) Jos Enrique Gallardo, Mark P Jones,
- * Alastair Reid and the Yale Haskell Group 1994-99, and is distributed as
- * Open Source software under the Artistic License; see the file "Artistic"
- * that is included in the distribution for details.
+ * The Hugs 98 system is Copyright (c) Jos Enrique Gallardo, Mark P Jones,
+ * Alastair Reid, the Yale Haskell Group, and the OGI School of
+ * Science & Engineering at OHSU, 1994-2003, All rights reserved.  It is
+ * distributed as free software under the license in the file "License",
+ * which is included in the distribution.
  *
  * This file is included in "Winhugs.c"
  * ------------------------------------------------------------------------*/
 
-
-
 /* --------------------------------------------------------------------------
  * Browse dialog boxes:
  * ------------------------------------------------------------------------*/
@@ -304,7 +303,7 @@ LRESULT CALLBACK BrowseClassesDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM
 	      currClass = (Class) SendDlgItemMessage(hDlg, LB_CLASS ,LB_GETITEMDATA, SendDlgItemMessage(hDlg, LB_CLASS ,LB_GETCURSEL, 0, 0L), 0L);
 
 	      currClass = (Class) SendDlgItemMessage(hDlg, LB_CLASS ,LB_GETITEMDATA, SendDlgItemMessage(hDlg, LB_CLASS ,LB_GETCURSEL, 0, 0L), 0L);
-	      setLastEdit(scriptName[scriptThisClass(currClass)], cclass(currClass).line);
+	      setLastEdit(getScriptName(scriptThisClass(currClass)), cclass(currClass).line);
 	      runEditor();
 	      
 	      
@@ -322,7 +321,7 @@ LRESULT CALLBACK BrowseClassesDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM
 	      currInst = (Inst) SendDlgItemMessage(hDlg, LB_INSTANCES, LB_GETITEMDATA, SendDlgItemMessage(hDlg, LB_INSTANCES ,LB_GETCURSEL, 0, 0L), 0L);
 
 	      /* Find instance module */
-	      setLastEdit(scriptName[scriptThisInst(currInst)], inst(currInst).line);
+	      setLastEdit(getScriptName(scriptThisInst(currInst)), inst(currInst).line);
 	      runEditor();
 	      
 	    }
@@ -345,6 +344,8 @@ LRESULT CALLBACK BrowseClassesDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM
 	  break;
 
 	case IDCANCEL: /* Close dialog */
+	    EndDialog(hDlg, FALSE);
+	    return TRUE;
 	case IDOK:
 	  EndDialog(hDlg, TRUE);
 	  return TRUE;
@@ -596,7 +597,7 @@ LRESULT CALLBACK BrowseNamesDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM l
 	      n = nth(theName, namesList);      
 
 	      if (!name(n).primDef) {
-		setLastEdit(scriptName[scriptThisName(n)], name(n).line);
+		setLastEdit(getScriptName(scriptThisName(n)), name(n).line);
 		runEditor();
 	      }
 	      else {
@@ -629,6 +630,8 @@ LRESULT CALLBACK BrowseNamesDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM l
 	    break;
 
 	case IDCANCEL: /* Close dialog */
+          EndDialog(hDlg, FALSE);
+	  return TRUE;
 	case IDOK:
 	  EndDialog(hDlg, TRUE);
 	  return TRUE;
@@ -984,7 +987,7 @@ LRESULT CALLBACK BrowseTyconsDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM
 	      tc = nth(TheTycon,tyconList);
 
 	      if (isTycon(tc) && tycon(tc).line) {
- 	        setLastEdit(scriptName[scriptThisTycon(tc)], tycon(tc).line);
+ 	        setLastEdit(getScriptName(scriptThisTycon(tc)), tycon(tc).line);
 	        runEditor();
 	      }
 	      else {
@@ -1003,7 +1006,7 @@ LRESULT CALLBACK BrowseTyconsDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM
 	      currInst = (Inst) SendDlgItemMessage(hDlg, LB_TYCONSINST, LB_GETITEMDATA, SendDlgItemMessage(hDlg, LB_TYCONSINST ,LB_GETCURSEL, 0, 0L), 0L);
 
 	      /* Find instance module */
-	      setLastEdit(scriptName[scriptThisInst(currInst)], inst(currInst).line);
+	      setLastEdit(getScriptName(scriptThisInst(currInst)), inst(currInst).line);
 	      runEditor();
 	    }
 	    break;
@@ -1053,9 +1056,11 @@ LRESULT CALLBACK BrowseTyconsDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM
 	    break;
 
 	case IDCANCEL:  /* Close dialog */
+	    EndDialog(hDlg, FALSE);
+	    return TRUE;
 	case IDOK:
-	  EndDialog(hDlg, TRUE);
-	  return TRUE;
+	    EndDialog(hDlg, TRUE);
+	    return TRUE;
 
 	default:
 	  return TRUE;
@@ -1079,7 +1084,7 @@ LRESULT CALLBACK BrowseTyconsDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM
 static VOID    local setClassBrowserSize     Args((Void));
 static VOID    local doCreate_Classes        Args((HWND));
 static VOID    local doDestroy_Classes       Args((Void));
-static Void    local doMove_Classes          Args((INT,INT));
+static Void    local doMove_Classes          Args((HWND,INT,INT));
 static Void    local doSize_Classes          Args((HWND,INT,INT));
 static Void    local doPaint_Classes         Args((HWND));
 static Void    local setOffset_Classes       Args((HWND,INT,INT));
@@ -1276,9 +1281,13 @@ static VOID local setClassBrowserSize() {
   cBrowse.width  = width  + 2*GetSystemMetrics(SM_CXFRAME);
   cBrowse.height = height + 2*GetSystemMetrics(SM_CYFRAME)
 			 + GetSystemMetrics(SM_CYCAPTION);
-  MoveWindow (hWndClasses, cBrowse.ClassesTopX, cBrowse.ClassesTopY,
-	      cBrowse.width  + GetSystemMetrics(SM_CXVSCROLL)+10,
-	      cBrowse.height + GetSystemMetrics(SM_CYHSCROLL)+10, TRUE);
+  SetWindowPos (hWndClasses, 
+		NULL,
+		0,
+		0,
+		cBrowse.width  + GetSystemMetrics(SM_CXVSCROLL)+10,
+		cBrowse.height + GetSystemMetrics(SM_CYHSCROLL)+10,
+		SWP_NOMOVE | SWP_NOACTIVATE);
 }
 
 static Void local doGetMinMaxInfo_Classes(MINMAXINFO FAR*lpmmi)
@@ -1485,12 +1494,15 @@ static Void local doDestroy_Classes() {
    hWndClasses=NULL;
 }
 
-static Void local doMove_Classes(INT x, INT y) {
-   /* We must subtract as it gets client position */
-   cBrowse.ClassesTopX = x - GetSystemMetrics(SM_CXFRAME);
-   cBrowse.ClassesTopY = y - GetSystemMetrics(SM_CYCAPTION)
-			   - GetSystemMetrics(SM_CYFRAME)
-			   + GetSystemMetrics(SM_CYBORDER);
+static Void local doMove_Classes(HWND hWnd, INT x, INT y) {
+   /* WM_MOVE's coords are for the upper-left of the client area;
+      we want the window's upper-left screen coords, so
+      just use GetWindowRect() to get at this. */
+    RECT r;
+
+    GetWindowRect(hWndClasses,&r);
+    cBrowse.ClassesTopX = r.left;
+    cBrowse.ClassesTopY = r.top;
 }
 
 static Void local doSize_Classes(HWND hWnd, INT width, INT height)
@@ -1537,7 +1549,7 @@ LRESULT CALLBACK ClassesWndProc (HWND hWnd, UINT msg, WPARAM wParam, LPARAM lPar
     doSize_Classes(hWnd, (INT) LOWORD(lParam), (INT) HIWORD(lParam));
     break;
   case WM_MOVE: 
-    doMove_Classes((INT) LOWORD(lParam), (INT) HIWORD(lParam));
+    doMove_Classes(hWnd, (INT) LOWORD(lParam), (INT) HIWORD(lParam));
     break;
   case WM_HSCROLL: 
     switch(LOWORD(wParam)) {
@@ -1599,6 +1611,7 @@ static VOID local DrawClassesHierarchy(VOID)
  if (hWndClasses) { /* If window exists keep its position */
    GetWindowRect(hWndClasses, &rWindow);
    DestroyWindow(hWndClasses);
+   
  } else {
    GetWindowRect(hActiveWindow, &rActive);
    rWindow.top  = rActive.top + 50;
@@ -1628,5 +1641,3 @@ static VOID local DrawClassesHierarchy(VOID)
 
  return;
 }
-
-
