@@ -1,29 +1,28 @@
 /* --------------------------------------------------------------------------
  * WinHint.h:	Jos Enrique Gallardo Ruiz, Feb 1999
  *
- * Hugs for Windows is Copyright (c) Jos Enrique Gallardo, Mark P Jones,
- * Alastair Reid and the Yale Haskell Group 1994-99, and is distributed as
- * Open Source software under the Artistic License; see the file "Artistic"
- * that is included in the distribution for details.
+ * The Hugs 98 system is Copyright (c) Jos Enrique Gallardo, Mark P Jones,
+ * Alastair Reid, the Yale Haskell Group, and the OGI School of
+ * Science & Engineering at OHSU, 1994-2003, All rights reserved.  It is
+ * distributed as free software under the license in the file "License",
+ * which is included in the distribution.
  *
  * This file contains the Header file for a hint window definition
  * ------------------------------------------------------------------------*/
-
-#define __WINHINT_H
+#ifndef __WINHINT_H__
+#define __WINHINT_H__
 
 #define MAXLNG	64
 
-
-
-
 typedef struct tagHINT {
   CHAR		HintStr[MAXLNG];	/* The message on the hint */
 } HINT;
 
 typedef HINT *HHINT;
 
-
 /* Functions defined in WinTip.c that are exported */
 HWND 	       HintCreateWindow	   	(HINSTANCE, HWND);
 BOOL 	       HintRegisterClass   	(HINSTANCE);
 
+#endif /* __WINHINT_H__ */
+
