@@ -1,26 +1,29 @@
 /* --------------------------------------------------------------------------
  * WinUtils.h:	Jos Enrique Gallardo Ruiz, Feb 1999
-  *
- * Hugs for Windows is Copyright (c) Jos Enrique Gallardo, Mark P Jones,
- * Alastair Reid and the Yale Haskell Group 1994-99, and is distributed as
- * Open Source software under the Artistic License; see the file "Artistic"
- * that is included in the distribution for details.
+ *
+ * The Hugs 98 system is Copyright (c) Jos Enrique Gallardo, Mark P Jones,
+ * Alastair Reid, the Yale Haskell Group, and the OGI School of
+ * Science & Engineering at OHSU, 1994-2003, All rights reserved.  It is
+ * distributed as free software under the license in the file "License",
+ * which is included in the distribution.
  *
  * This file contains the Header file for some common functions
  * ------------------------------------------------------------------------*/
-
-#define __WINUTILS_H
+#ifndef __WINUTILS_H__
+#define __WINUTILS_H__
 
 VOID 	CenterDialogInParent 	(HWND);
 VOID	SetDialogFont		(HWND, HFONT);
 BOOL 	CheckExt 		(LPCSTR, LPCSTR);
 VOID 	DrawTransparentBitmap	(HDC, HBITMAP, UINT, UINT, COLORREF);
 VOID 	DrawBitmap 		(HDC, HBITMAP, UINT, UINT);
-VOID 	ExecDialog 		(HINSTANCE, WORD, WNDPROC);
+BOOL 	ExecDialog 		(HINSTANCE, WORD, WNDPROC);
 VOID	FullPath		(LPSTR, LPCSTR);
 VOID	MapBitmap 		(HBITMAP, COLORREF, COLORREF);
 HBITMAP LoadMappedBitmap	(HINSTANCE, LPCSTR);
 HBITMAP ResizeBitmap            (HBITMAP, UINT, UINT);
 INT 	SetWorkingDir 		(LPCSTR);
 VOID 	StrReplace		(CHAR*, CHAR*, CHAR*, CHAR*);
-VOID    ShortFileName           (CHAR *, CHAR *);
\ No newline at end of file
+VOID    ShortFileName           (CHAR *, CHAR *);
+
+#endif /* __WINUTILS_H__ */
